@@ -17,7 +17,7 @@
 
 import re
 import struct
-from uuid import UUID
+from decimal import Decimal
 
 import cql
 
@@ -27,17 +27,31 @@ if hasattr(struct, 'Struct'): # new in Python 2.5
    _have_struct = True
    _long_packer = struct.Struct('>q')
    _int32_packer = struct.Struct('>i')
+   _float_packer = struct.Struct('>f')
+   _double_packer = struct.Struct('>d')
 else:
     _have_struct = False
 
+try:
+    from uuid import UUID  # new in Python 2.5
+except ImportError:
+    class UUID:
+        def __init__(self, bytes):
+            self.bytes = bytes
+
 _param_re = re.compile(r"(?<!strategy_options)(?<!\\)(:[a-zA-Z_][a-zA-Z0-9_]*)", re.M)
 
 BYTES_TYPE = "org.apache.cassandra.db.marshal.BytesType"
 ASCII_TYPE = "org.apache.cassandra.db.marshal.AsciiType"
+BOOLEAN_TYPE = "org.apache.cassandra.db.marshal.BooleanType"
+DATE_TYPE = "org.apache.cassandra.db.marshal.DateType"
+DECIMAL_TYPE = "org.apache.cassandra.db.marshal.DecimalType"
 UTF8_TYPE = "org.apache.cassandra.db.marshal.UTF8Type"
 INT32_TYPE = "org.apache.cassandra.db.marshal.Int32Type"
 INTEGER_TYPE = "org.apache.cassandra.db.marshal.IntegerType"
 LONG_TYPE = "org.apache.cassandra.db.marshal.LongType"
+FLOAT_TYPE = "org.apache.cassandra.db.marshal.FloatType"
+DOUBLE_TYPE = "org.apache.cassandra.db.marshal.DoubleType"
 UUID_TYPE = "org.apache.cassandra.db.marshal.UUIDType"
 LEXICAL_UUID_TYPE = "org.apache.cassandra.db.marshal.LexicalType"
 TIME_UUID_TYPE = "org.apache.cassandra.db.marshal.TimeUUIDType"
@@ -63,35 +77,91 @@ def marshal(term):
 def unmarshal_noop(bytestr):
     return bytestr
 
+def unmarshal_bool(bytestr):
+    if not bytestr:
+        return None
+    return bool(ord(bytestr[0]))
+
 def unmarshal_utf8(bytestr):
     return bytestr.decode("utf8")
 
 if _have_struct:
     def unmarshal_int32(bytestr):
+        if not bytestr:
+            return None
         return _int32_packer.unpack(bytestr)[0]
 else:
     def unmarshal_int32(bytestr):
+        if not bytestr:
+            return None
         return struct.unpack(">i", bytestr)[0]
 
 def unmarshal_int(bytestr):
+    if not bytestr:
+        return None
     return decode_bigint(bytestr)
 
 if _have_struct:
     def unmarshal_long(bytestr):
+        if not bytestr:
+            return None
         return _long_packer.unpack(bytestr)[0]
 else:
     def unmarshal_long(bytestr):
+        if not bytestr:
+            return None
         return struct.unpack(">q", bytestr)[0]
 
+if _have_struct:
+    def unmarshal_float(bytestr):
+        if not bytestr:
+            return None
+        return _float_packer.unpack(bytestr)[0]
+else:
+    def unmarshal_long(bytestr):
+        if not bytestr:
+            return None
+        return struct.unpack(">f", bytestr)[0]
+
+if _have_struct:
+    def unmarshal_double(bytestr):
+        if not bytestr:
+            return None
+        return _double_packer.unpack(bytestr)[0]
+else:
+    def unmarshal_long(bytestr):
+        if not bytestr:
+            return None
+        return struct.unpack(">d", bytestr)[0]
+
+def unmarshal_date(bytestr):
+    if not bytestr:
+        return None
+    return unmarshal_long(bytestr) / 1000.0
+
+def unmarshal_decimal(bytestr):
+    if not bytestr:
+        return None
+    scale = unmarshal_int32(bytestr[:4])
+    unscaled = decode_bigint(bytestr[4:])
+    return Decimal('%de%d' % (unscaled, -scale))
+
 def unmarshal_uuid(bytestr):
+    if not bytestr:
+        return None
     return UUID(bytes=bytestr)
 
 unmarshallers = {BYTES_TYPE:          unmarshal_noop,
                  ASCII_TYPE:          unmarshal_noop,
+                 BOOLEAN_TYPE:        unmarshal_bool,
+                 DATE_TYPE:           unmarshal_date,
+                 DECIMAL_TYPE:        unmarshal_decimal,
                  UTF8_TYPE:           unmarshal_utf8,
                  INT32_TYPE:          unmarshal_int32,
                  INTEGER_TYPE:        unmarshal_int,
                  LONG_TYPE:           unmarshal_long,
+                 FLOAT_TYPE:          unmarshal_float,
+                 DOUBLE_TYPE:         unmarshal_double,
                  UUID_TYPE:           unmarshal_uuid,
                  LEXICAL_UUID_TYPE:   unmarshal_uuid,
                  TIME_UUID_TYPE:      unmarshal_uuid,
